package controller

import (
	"electricity_bill_calc/exceptions"
	"electricity_bill_calc/response"
	"electricity_bill_calc/security"
	"electricity_bill_calc/service"
	"net/http"

	"github.com/gofiber/fiber/v2"
)

func InitializeGodModeController(router *fiber.App) {
	gmR := router.Group("/gm")
	{
		gmR.Delete("/report/:rid/summary", security.SingularityAuthorize, gmResetReportSummary)
		gmR.Delete("/report/:rid/maintenance", security.SingularityAuthorize, gmResetReportMaintenance)
		gmR.Delete("/report/:rid/meters", security.SingularityAuthorize, gmResetReportEndUserRecord)
		gmR.Post("/report/:rid/meters", security.SingularityAuthorize, gmResynchronizeReportEndUserRecord)
		gmR.Delete("/report/:rid", security.SingularityAuthorize, gmResetReport)
		gmR.Delete("/report/:rid/force", security.SingularityAuthorize, gmDeleteReport)
		gmR.Delete("/park/:pid/maintenance/:mid", security.SingularityAuthorize, gmDeleteSpecificMaintenance)
		gmR.Delete("/park/:pid/maintenance", security.SingularityAuthorize, gmDeleteAllMaintenance)
		gmR.Delete("/park/:pid/meters", security.SingularityAuthorize, gmDeleteAllMeters)
		gmR.Delete("/park/:pid/force", security.SingularityAuthorize, gmDeletePark)
		gmR.Delete("/enterprise/:uid/force", security.SingularityAuthorize, gmDeleteUser)
	}
}

func gmResetReportSummary(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.ClearReportSummary(requestReportId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功重置指定报表的园区总览部分。")
	}
	return result.Success("指定报表的园区总览已经重置。")
}

func gmResetReportMaintenance(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.ClearReportMaintenances(requestReportId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功重置指定报表的配电维护费部分。")
	}
	return result.Success("指定报表的配电维护费已经重置。")
}

func gmResynchronizeReportEndUserRecord(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.ResynchronizeEndUser(requestReportId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功重置指定报表的抄表记录基本档案。")
	}
	return result.Success("指定报表的抄表记录基本档案已经重新同步。")
}

func gmResetReportEndUserRecord(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.ResetEndUserRegisterRecords(requestReportId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功重置指定报表的抄表记录部分。")
	}
	return result.Success("指定报表的抄表记录已经重置。")
}

func gmResetReport(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.ResetReport(requestReportId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功重置指定报表。")
	}
	return result.Success("指定报表已经重置。")
}

func gmDeleteReport(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestReportId := c.Params("rid")
	done, err := service.GodModeService.DeleteReport(requestReportId)
	if err != nil {
		if ipErr, ok := err.(exceptions.ImproperOperateError); ok {
			return result.NotAccept(ipErr.Message)
		} else {
			return result.Error(http.StatusInternalServerError, err.Error())
		}
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除指定报表。")
	}
	return result.Success("指定报表已经删除。")
}

func gmDeleteSpecificMaintenance(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestParkId := c.Params("pid")
	requestMaintenanceId := c.Params("mid")
	done, err := service.GodModeService.RemoveSpecificMaintenance(requestParkId, requestMaintenanceId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除指定的维护费用记录。")
	}
	return result.Success("指定维护费用记录已经删除。")
}

func gmDeleteAllMaintenance(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestParkId := c.Params("pid")
	done, err := service.GodModeService.RemoveAllMaintenance(requestParkId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除全部维护费用记录。")
	}
	return result.Success("全部维护费用记录已经删除。")
}

func gmDeleteAllMeters(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestParkId := c.Params("pid")
	done, err := service.GodModeService.RemoveAllMeters(requestParkId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除全部终端表计档案记录。")
	}
	return result.Success("全部终端表计档案记录已经删除。")
}

func gmDeletePark(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestParkId := c.Params("pid")
	done, err := service.GodModeService.RemovePark(requestParkId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除指定的园区。")
	}
	return result.Success("指定的园区已经删除。")
}

func gmDeleteUser(c *fiber.Ctx) error {
	result := response.NewResult(c)
	requestUserId := c.Params("uid")
	done, err := service.GodModeService.DeleteUser(requestUserId)
	if err != nil {
		return result.Error(http.StatusInternalServerError, err.Error())
	}
	if !done {
		return result.Error(http.StatusInternalServerError, "未能成功删除指定的用户。")
	}
	return result.Success("指定的用户及其关联信息已经删除。")
}
